@@ -489,14 +489,11 @@ class TestAtoms(BaseTest):
         self.assertEqual(cp.sum(Variable((2, 3)), axis=0, keepdims=False).shape, (3,))
         self.assertEqual(cp.sum(Variable((2, 3)), axis=1).shape, (2,))
 
-        # Invalid axis.
-        with self.assertRaises(Exception) as cm:
+        # Invalid axis - now raises ValueError with context
+        with self.assertRaises(ValueError):
             cp.sum(self.x, axis=4)
-        self.assertEqual(str(cm.exception),
-                        "axis 4 is out of bounds for array of dimension 1")
-        with self.assertRaises(Exception) as cm:
+        with self.assertRaises(ValueError):
             cp.sum(Variable(2), axis=1).shape
-        self.assertEqual(str(cm.exception), "axis 1 is out of bounds for array of dimension 1")
 
         A = sp.eye_array(3)
         self.assertEqual(cp.sum(A).value, 3)
@@ -541,34 +538,56 @@ class TestAtoms(BaseTest):
             entries.append(self.x[i])
         atom = cp.vstack(entries)
         self.assertEqual(atom.shape, (2, 1))
-        # self.assertEqual(atom[1,0].name(), "vstack(x[0,0], x[1,0])[1,0]")
 
-        with self.assertRaises(Exception) as cm:
+        with self.assertRaises(ValueError):
             cp.vstack([self.C, 1])
-        self.assertEqual(str(cm.exception),
-                         "All the input dimensions except for axis 0 must match exactly.")
 
-        with self.assertRaises(Exception) as cm:
+        with self.assertRaises(ValueError):
             cp.vstack([self.x, Variable(3)])
-        self.assertEqual(str(cm.exception),
-                         "All the input dimensions except for axis 0 must match exactly.")
 
-        with self.assertRaises(TypeError) as cm:
+        with self.assertRaises(TypeError):
             cp.vstack()
 
         # Test scalars with variables of shape (1,)
         expr = cp.vstack([2, Variable((1,))])
         self.assertEqual(expr.shape, (2, 1))
 
+    def test_hstack(self) -> None:
+        atom = cp.hstack([self.x, self.y, self.x])
+        self.assertEqual(atom.name(), "Hstack(x, y, x)")
+        self.assertEqual(atom.shape, (6,))
+
+        atom = cp.hstack([self.A, self.B])
+        self.assertEqual(atom.name(), "Hstack(A, B)")
+        self.assertEqual(atom.shape, (2, 4))
+        
+        # Extracting columns produces 1D arrays, so hstack concatenates to (4,)
+        entries = []
+        for i in range(self.A.shape[1]):
+            entries.append(self.A[:, i])
+        atom = cp.hstack(entries)
+        self.assertEqual(atom.shape, (4,))
+        
+        with self.assertRaises(ValueError):
+            cp.hstack([self.C, self.A])
+
+        with self.assertRaises(ValueError):
+            cp.hstack([self.A, self.x])
+
+        with self.assertRaises(TypeError):
+            cp.hstack()
+
+        expr = cp.hstack([2, Variable((1,))])
+        self.assertEqual(expr.shape, (2,))
+
     def test_concatenate(self):
         atom = cp.concatenate([self.x, self.y], axis=0)
         self.assertEqual(atom.name(), "Concatenate(x, y, 0)")
         self.assertEqual(atom.shape, (4,))  # (2 vectors are concatenated on axis 0)
 
         with self.assertRaises(ValueError):
-            # x and y are 1D arrays, so they can't be concatenated on axis 1
-            atom = cp.concatenate([self.x, self.y], axis=1)
-        # Expected ValueError due to invalid axis for 1D arrays
+            # NumPy raises AxisError for invalid axis, converted to ValueError
+            cp.concatenate([self.x, self.y], axis=1)
 
         atom = cp.concatenate([self.A, self.C], axis=None)
         self.assertEqual(atom.shape, (10,))
@@ -577,8 +596,8 @@ class TestAtoms(BaseTest):
         self.assertEqual(atom.shape, (5, 2))
 
         with self.assertRaises(ValueError):
-            atom = cp.concatenate([self.A, self.C], axis=1)
-        # Expected ValueError due to mismatched dimensions along dimension 0
+            # A is (2,2) and C is (3,2) - can't concatenate on axis 1
+            cp.concatenate([self.A, self.C], axis=1)
 
         atom = cp.concatenate([self.A, self.B], axis=1)
         self.assertEqual(atom.shape, (2, 4))
@@ -940,18 +959,13 @@ class TestAtoms(BaseTest):
         with self.assertRaises(Exception) as cm:
             cp.sum_largest(self.x, -1)
         self.assertEqual(str(cm.exception),
-                         "Second argument must be a positive integer.")
+                         "Second argument must be a positive number.")
 
         with self.assertRaises(Exception) as cm:
             cp.lambda_sum_largest(self.x, 2.4)
         self.assertEqual(str(cm.exception),
                          "First argument must be a square matrix.")
 
-        with self.assertRaises(Exception) as cm:
-            cp.lambda_sum_largest(Variable((2, 2)), 2.4)
-        self.assertEqual(str(cm.exception),
-                         "Second argument must be a positive integer.")
-
         with self.assertRaises(ValueError) as cm:
             cp.lambda_sum_largest([[1, 2], [3, 4]], 2).value
         self.assertEqual(str(cm.exception),
@@ -976,6 +990,17 @@ class TestAtoms(BaseTest):
         copy = atom.copy()
         self.assertTrue(type(copy) is type(atom))
 
+        # Test lambda_sum_largest with float k
+        A = np.array([[2.0, 0.0], [0.0, 1.0]])  # Eigenvalues: [2, 1]
+        expr = cp.lambda_sum_largest(A, 1.5)
+        expected = 2.0 + 0.5 * 1.0  # Largest + 0.5 * second largest
+        self.assertAlmostEqual(expr.value, expected)
+
+        # Test lambda_sum_smallest with float k
+        expr = cp.lambda_sum_smallest(A, 1.3)
+        expected = 1.0 + 0.3 * 2.0  # Smallest + 0.3 * second smallest
+        self.assertAlmostEqual(expr.value, expected)
+
         # Check that sum_largest is PWL so can be canonicalized as a QP.
         atom = cp.sum_largest(self.x, 2)
         assert atom.is_pwl()
@@ -988,23 +1013,49 @@ class TestAtoms(BaseTest):
             prev_idx = np.argsort(-v)[:i]
             self.assertAlmostEqual(expr.value, v[prev_idx].sum())
 
+        # Test with float k values
+        v = np.array([5.0, 3.0, 8.0, 1.0, 6.0])
+        x = Constant(v)
+        # k=2.5 should give sum of 2 largest (8 + 6 = 14) + 0.5 * 3rd largest (0.5 * 5 = 2.5) = 16.5
+        expr = cp.sum_largest(x, 2.5)
+        expected = 8.0 + 6.0 + 0.5 * 5.0
+        self.assertAlmostEqual(expr.value, expected)
+
+        # k=1.7 should give largest (8) + 0.7 * 2nd largest (0.7 * 6 = 4.2) = 12.2
+        expr = cp.sum_largest(x, 1.7)
+        expected = 8.0 + 0.7 * 6.0
+        self.assertAlmostEqual(expr.value, expected)
+
+        # k=0.3 should give 0.3 * largest (0.3 * 8 = 2.4)
+        expr = cp.sum_largest(x, 0.3)
+        expected = 0.3 * 8.0
+        self.assertAlmostEqual(expr.value, expected)
+
     def test_sum_smallest(self) -> None:
         """Test the sum_smallest atom and related atoms.
         """
         with self.assertRaises(Exception) as cm:
             cp.sum_smallest(self.x, -1)
         self.assertEqual(str(cm.exception),
-                         "Second argument must be a positive integer.")
-
-        with self.assertRaises(Exception) as cm:
-            cp.lambda_sum_smallest(Variable((2, 2)), 2.4)
-        self.assertEqual(str(cm.exception),
-                         "Second argument must be a positive integer.")
+                         "Second argument must be a positive number.")
 
         # Check that sum_smallest is PWL so can be canonicalized as a QP.
         atom = cp.sum_smallest(self.x, 2)
         assert atom.is_pwl()
 
+        # Test with float k values
+        v = np.array([5.0, 3.0, 8.0, 1.0, 6.0])
+        x = Constant(v)
+        # k=2.5 should give sum of 2 smallest (1 + 3 = 4) + 0.5 * 3rd smallest (0.5 * 5 = 2.5) = 6.5
+        expr = cp.sum_smallest(x, 2.5)
+        expected = 1.0 + 3.0 + 0.5 * 5.0
+        self.assertAlmostEqual(expr.value, expected)
+
+        # k=1.7 should give smallest (1) + 0.7 * 2nd smallest (0.7 * 3 = 2.1) = 3.1
+        expr = cp.sum_smallest(x, 1.7)
+        expected = 1.0 + 0.7 * 3.0
+        self.assertAlmostEqual(expr.value, expected)
+
     def test_cvar(self) -> None:
         """Test the cvar atom and its use in a linear program."""
         # Check that CVaR is correctly computed
